/*
  Algorithms IE171- Lab11/Cryptography
  Author:  Menal Guzelsoy - 04/13 Spring 2004
*/

/*------------------------------------------------------------------------*/

#ifndef RSA_H
#define RSA_H

#include<iostream>
#include<fstream>
#include<string>
#include<map>

typedef unsigned long int INT;
using namespace std;

class RSA{
   
public:

   /* Constructors */
   RSA();
   /* Pass in the keyFile which will include your public and private 
      keys */
   RSA(ifstream &keyFile);

   /* Destructor */
   ~RSA(){}

   /* Generate public and private keys. You can use this function whenever
      you need to get/change your keys. It will publish the public key 
      whereas will write out the private key to a file -myKeys.txt by default 
   */
   void generateKeys();

   /* Encode the loaded text using your public key*/
   string encode(string textToCode);   

   /* Encode the loaded text using the given pubKey and block sizes*/
   string encode(string textToCode, pair<INT, INT> pubKey, int encSize=0, 
		 int decSize =0);

   /* Decode the loaded text using your private key */
   string decode(string textToCode);

   /* Decode the loaded text using the given private key and block sizes*/
   string decode(string textToCode, pair<INT, INT> priKey, int encSize=0, 
		 int decSize =0);

   /* Break the loaded encrypted text through pubKey */
   string breakCode(string textToCode, pair<INT, INT> pubKey, int encSize=0, 
		    int decSize =0);

   /* Setting functions */
   void setPublicKey(pair<INT, INT> pubKey){publicKey_ = pubKey;}
   void setPrivateKey(pair<INT, INT> priKey){privateKey_ = priKey;}
   void setBlockSizes(int encSize, int decSize)
   {encBlockSize_ = encSize;
    decBlockSize_ = decSize;}

   /* Query functions */
   pair<INT, INT> getPublicKey(){return publicKey_;}
   int getIntEqvofChar(char ch){return toInt_[ch];}
   char getCharEqvofInt(int i){return toChar_[i];}

private:

   /* Initialize the alphabet */
   void initialize_();

   /* Helper function used for decode() and breakCode() */
   string decode_(string textToCode, pair<INT, INT> priKey, int encSize=0, 
		  int decSize =0);

   /* Convert an int to its string equivalent */
   INT convertToInt_(string str);

   /* Convert an int to its charNum-size string equivalent */
   string convertToStr_(INT num, int charNum);
   
   /* Factorize the nA to its prime factors-assuming it is composed of two 
      primes */
   pair<INT, INT> factorize_(INT nA);
   
   /* Find the inverse of eA in mod phi*/
   INT getInverse_(INT eA, INT phi);

   /* Calculate val^power modulo mod */
   INT getPower_(INT val, INT power, INT mod);

   /* Generate a pair of primes */
   pair<INT,INT> generatePrimeNumbers_();

   /* Find a random number which is relatively prime to the primeFactors */
   INT getRandNumPrimeToPhi_(pair<INT, INT> primeFactors);

   /* Find the greatest common divisor of a and b */
   INT gcd_(INT a, INT b);

   /* Map the characters to integers */
   map<char, int> toInt_;

   /* Map the integers to characters */
   map<int, char> toChar_;

   /* The public key (eA & nA) */
   pair<INT, INT> publicKey_;

   /* The private key (dA & nA) */
   pair<INT, INT> privateKey_;

   /* Block size of encryption */
   int encBlockSize_;

   /* Block size of decryption */
   int decBlockSize_;

   /* Total number of the characters of the underlying alphabet */
   int N_;

};

#endif



